#include <avr/io.h>
#include "twi.h"
#include "rtc_lib.h"
#include "LCD_lib.h"
#include "char_code.h"

/* Inicialize RTC oscillator */
void rtc_init(void)
{
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(0x00);
	twiwrite(0x00);
	twistop();
}

/* Set Register pointer */
void set_reg_pointer(uint8_t reg)
{
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(reg);
	twiwrite(0x00);
	twistop();
}

/* Set Control Register */
void rtc_control_reg(uint8_t out,uint8_t sqwe,uint8_t rs1,uint8_t rs0)
{
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Control_Addr);
	twiwrite(out<<7 | sqwe<<4 | rs1<<1 | rs0<<0);
	twistop();
}


/* Set Time */
// h12_h24 = 1 - 12 hours mode
// PM_AM = 1 - PM 
void rtc_set_time(uint8_t seconds, uint8_t minutes, uint8_t hours, uint8_t h12_h24, uint8_t PM_AM)
{
	//seconds
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Seconds_Addr);
	twiwrite(seconds);
	twistop();
	
	//minutes
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Minutes_Addr);
	twiwrite(minutes);
	twistop();
	//hours
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Hours_Addr);
	twiwrite(hours | h12_h24<<6 | PM_AM<<5);
	twistop();
	
}

/* Set Date */
void rtc_set_date(uint8_t day, uint8_t date, uint8_t month, uint8_t year)
{
	//day
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Day_Addr);
	twiwrite(day);
	twistop();
	//date
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Date_Addr);
	twiwrite(date);
	twistop();
	//month
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Month_Addr);
	twiwrite(month);
	twistop();
	//year
	twistart();
	twiwrite(Dev_Addr);
	twiwrite(Year_Addr);
	twiwrite(year);
	twistop();
}

/* Get Time */
// Function get time as string and put it into LCD
// put_sec = 1 - show seconds
void rtc_get_time(uint8_t put_sec)
{
uint8_t temp_s;
uint8_t temp_m;
uint8_t temp_h;

uint8_t time_s[2];
uint8_t time_m[2];
uint8_t time_h[2];

uint8_t sec;

	if(put_sec) sec=1; else sec=0;

	//read time
	set_reg_pointer(0x3F);
	twistart();
	twiwrite(0xD1);
	temp_s = twiread(ACK);
	temp_m = twiread(ACK);
	temp_h = twiread(NOACK);
	twistop();
	
	//12-24
	if(!(temp_h&0x40))
	{
		itoa(temp_h,time_h,16);
		if(temp_h<0x10) {LCDsendChar('0'); LCDstring(time_h,1);} else {LCDstring(time_h,2);}
		LCDsendChar(':');
		itoa(temp_m,time_m,16);
		if(temp_m<0x10) {LCDsendChar('0'); LCDstring(time_m,1);} else {LCDstring(time_m,2);}

		if(sec)
		{
			LCDsendChar(':');
			itoa(temp_s,time_s,16);
			if(temp_s<0x10) {LCDsendChar('0'); LCDstring(time_s,1);} else LCDstring(time_s,2);
		}  
	}
	else
	{
		itoa((temp_h&0x1F),time_h,16);
		if((temp_h&0x1F)<0x10) {LCDsendChar('0'); LCDstring(time_h,1);} else LCDstring(time_h,2);
		LCDsendChar(':');
		itoa(temp_m,time_m,16);
		if(temp_m<0x10) {LCDsendChar('0'); LCDstring(time_m,1);} else LCDstring(time_m,2);

		if(sec)
		{
			LCDsendChar(':');
			itoa(temp_s,time_s,16);
			if(temp_s<0x10) {LCDsendChar('0'); LCDstring(time_s,1);} else LCDstring(time_s,2);
		}

		if((temp_h&0x20)) LCDstring(" PM",3); else LCDstring(" AM",3);    
	}

}

/* Get Day */
// Function get day as string and put it into LCD
void rtc_get_day(void)
{
uint8_t day;

	//read day
	set_reg_pointer(0x3F);
	twistart();
	twiwrite(0xD1);
	twiread(ACK);
	twiread(ACK);
	twiread(ACK);
	day = twiread(NOACK); 
	twistop();

	switch(day)
	{
	    case 0x01 : LCDstring("Mon",3);
        case 0x02 : LCDstring("Tue",3);
        case 0x03 : LCDstring("Wed",3);
        case 0x04 : LCDstring("Thu",3);
        case 0x05 : LCDstring("Fri",3);
		case 0x06 : LCDstring("Sat",3);
		case 0x07 : LCDstring("Sun",3);
	}

}

/* Get Date */
// Function get date as string and put it into LCD
void rtc_get_date(void)
{
uint8_t temp_day;
uint8_t temp_month;
uint8_t temp_year;

uint8_t day[2];
uint8_t month[2];
uint8_t year[2];

	//read day
	set_reg_pointer(0x3F);
	twistart();
	twiwrite(0xD1);
	twiread(ACK);
	twiread(ACK);
	twiread(ACK);
	twiread(ACK);
	temp_day = twiread(ACK);
	temp_month = twiread(ACK);
	temp_year = twiread(NOACK);  
	twistop();

	itoa(temp_day,day,16);
	if(temp_day<0x10) {LCDsendChar('0'); LCDstring(day,1);} else {LCDstring(day,2);}
	LCDsendChar('/');
    itoa(temp_month,month,16);
	if(temp_month<0x10) {LCDsendChar('0'); LCDstring(month,1);} else {LCDstring(month,2);}
	LCDsendChar('/');
	itoa(temp_year,year,16);
	if(temp_year<0x10) {LCDsendChar('0'); LCDstring(year,1);} else {LCDstring(year,2);}
}
