#include "usart.h"
#include "ascii_code.h"
#include <avr/io.h>


/* Inicjalizacja */
void UART_init(unsigned int ubrr) 
{
  /* 4800 bit/s */
  UBRR1H = (unsigned char)(ubrr>>8);
  UBRR1L = (unsigned char)ubrr;
  UCSR1B = _BV(TXCIE1)|_BV(RXCIE1)|_BV(RXEN1)|_BV(TXEN1);
  /* 8-bit dane, 1-bit stopu, prak bitu parzystosci */
  UCSR1C = (0<<USBS1)|(3<<UCSZ10);// (1<<URSEL1)|           
}

/* Transmisja */
void UART_putchar(char c)
{ 
  UDR1 = c;                                // wpisz c do rejestru UDR
  loop_until_bit_is_set(UCSR1A,TXC1);       // czekaj na zakonczenie transmisji
  UCSR1A = _BV(TXC1);                       // ustaw bit TXC w rej. USR
}

/* Odbir */
char UART_getchar(void)
{
  loop_until_bit_is_set(UCSR1A,RXC1);        // czekaj na zakonczenie odbioru
  UCSR1A = _BV(RXC1);                       // skasuj bit RXC w rej. USR
  return UDR1;                              // zwroc zawartosc rejestru UDR
}

/* Transmisja teksu */
void UART_string(uint8_t* data, uint8_t nBytes)	
{
register uint8_t i;

	if (!data) return;

	for(i=0; i<nBytes; i++)
	{
		UART_putchar(data[i]);
	}
}

/* Przejscie do nastepnej linii */
void UART_nexline(void)
{
	UART_putchar(0x0D); //carriage return
}

/* Backspace */
void UART_backspace(uint8_t nBytes)
{
register uint8_t i;

	for(i=0; i<nBytes; i++)
	{
		UART_putchar(0x08); //backspace
	}	
}
